package com.sun.org.apache.xalan.internal.xsltc.dom;

import com.sun.org.apache.xalan.internal.xsltc.runtime.BasisLibrary;
import com.sun.org.apache.xml.internal.dtm.DTMAxisIterator;
import java.util.StringTokenizer;
import java.util.HashMap;
import com.sun.org.apache.xalan.internal.xsltc.DOMEnhancedForDTM;
import com.sun.org.apache.xalan.internal.xsltc.DOM;
import com.sun.org.apache.xalan.internal.xsltc.util.IntegerArray;
import java.util.Map;
import com.sun.org.apache.xml.internal.dtm.ref.DTMAxisIteratorBase;

public class KeyIndex extends DTMAxisIteratorBase
{
    private Map<String, IntegerArray> _index;
    private int _currentDocumentNode;
    private Map<Integer, Map> _rootToIndexMap;
    private IntegerArray _nodes;
    private DOM _dom;
    private DOMEnhancedForDTM _enhancedDOM;
    private int _markedPosition;
    private static final IntegerArray EMPTY_NODES;
    
    public KeyIndex(int dummy) {
        this._currentDocumentNode = -1;
        this._rootToIndexMap = new HashMap();
        this._nodes = null;
        this._markedPosition = 0;
    }
    
    @Override
    public void setRestartable(boolean flag) {
    }
    
    public void add(String value, int node, int rootNode) {
        if (this._currentDocumentNode != rootNode) {
            this._currentDocumentNode = rootNode;
            this._index = new HashMap();
            this._rootToIndexMap.put(Integer.valueOf(rootNode), this._index);
        }
        IntegerArray nodes = (IntegerArray)this._index.get(value);
        if (nodes == null) {
            nodes = new IntegerArray();
            this._index.put(value, nodes);
            nodes.add(node);
        }
        else if (node != nodes.at(nodes.cardinality() - 1)) {
            nodes.add(node);
        }
    }
    
    @Deprecated
    public void merge(KeyIndex other) {
        if (other == null) {
            return;
        }
        if (other._nodes != null) {
            if (this._nodes == null) {
                this._nodes = (IntegerArray)other._nodes.clone();
            }
            else {
                this._nodes.merge(other._nodes);
            }
        }
        return;
    }
    
    @Deprecated
    public void lookupId(Object value) {
        this._nodes = null;
        StringTokenizer values = new StringTokenizer((String)value, " \n\t");
        while (values.hasMoreElements()) {
            String token = (String)values.nextElement();
            IntegerArray nodes = (IntegerArray)this._index.get(token);
            if (nodes == null && this._enhancedDOM != null && this._enhancedDOM.hasDOMSource()) {
                nodes = this.getDOMNodeById(token);
            }
            if (nodes == null) {
                continue;
            }
            if (this._nodes == null) {
                nodes = (IntegerArray)nodes.clone();
                this._nodes = nodes;
            }
            else {
                this._nodes.merge(nodes);
            }
        }
    }
    
    public IntegerArray getDOMNodeById(String id) {
        IntegerArray nodes = null;
        if (this._enhancedDOM != null) {
            int ident = this._enhancedDOM.getElementById(id);
            if (ident != -1) {
                Integer root = Integer.valueOf(this._enhancedDOM.getDocument());
                Map<String, IntegerArray> index = (Map)this._rootToIndexMap.get(root);
                if (index == null) {
                    index = new HashMap();
                    this._rootToIndexMap.put(root, index);
                }
                else {
                    nodes = (IntegerArray)index.get(id);
                }
                if (nodes == null) {
                    nodes = new IntegerArray();
                    index.put(id, nodes);
                }
                nodes.add(this._enhancedDOM.getNodeHandle(ident));
            }
        }
        return nodes;
    }
    
    @Deprecated
    public void lookupKey(Object value) {
        IntegerArray nodes = (IntegerArray)this._index.get(value);
        this._nodes = ((nodes != null) ? ((IntegerArray)nodes.clone()) : null);
        this._position = 0;
    }
    
    @Override
    @Deprecated
    public int next() {
        if (this._nodes == null) {
            return -1;
        }
        return (this._position < this._nodes.cardinality()) ? this._dom.getNodeHandle(this._nodes.at(this._position++)) : -1;
    }
    
    public int containsID(int node, Object value) {
        String string = (String)value;
        int rootHandle = this._dom.getAxisIterator(19).setStartNode(node).next();
        Map<String, IntegerArray> index = (Map)this._rootToIndexMap.get(Integer.valueOf(rootHandle));
        StringTokenizer values = new StringTokenizer(string, " \n\t");
        while (values.hasMoreElements()) {
            String token = (String)values.nextElement();
            IntegerArray nodes = null;
            if (index != null) {
                nodes = (IntegerArray)index.get(token);
            }
            if (nodes == null && this._enhancedDOM != null && this._enhancedDOM.hasDOMSource()) {
                nodes = this.getDOMNodeById(token);
            }
            if (nodes != null && nodes.indexOf(node) >= 0) {
                return 1;
            }
            continue;
        }
        return 0;
    }
    
    public int containsKey(int node, Object value) {
        int rootHandle = this._dom.getAxisIterator(19).setStartNode(node).next();
        Map<String, IntegerArray> index = (Map)this._rootToIndexMap.get(Integer.valueOf(rootHandle));
        if (index != null) {
            IntegerArray nodes = (IntegerArray)index.get(value);
            return nodes != null && nodes.indexOf(node) >= 0?1:0;
        }
        return 0;
    }
    
    @Override
    @Deprecated
    public DTMAxisIterator reset() {
        this._position = 0;
        return this;
    }
    
    @Override
    @Deprecated
    public int getLast() {
        return (this._nodes == null) ? 0 : this._nodes.cardinality();
    }
    
    @Override
    @Deprecated
    public int getPosition() {
        return this._position;
    }
    
    @Override
    @Deprecated
    public void setMark() {
        this._markedPosition = this._position;
    }
    
    @Override
    @Deprecated
    public void gotoMark() {
        this._position = this._markedPosition;
    }
    
    @Override
    @Deprecated
    public DTMAxisIterator setStartNode(int start) {
        if (start == -1) {
            this._nodes = null;
        }
        else if (this._nodes != null) {
            this._position = 0;
        }
        return this;
    }
    
    @Override
    @Deprecated
    public int getStartNode() {
        return 0;
    }
    
    @Override
    @Deprecated
    public boolean isReverse() {
        return false;
    }
    
    @Override
    @Deprecated
    public DTMAxisIterator cloneIterator() {
        KeyIndex other = new KeyIndex(0);
        other._index = this._index;
        other._rootToIndexMap = this._rootToIndexMap;
        other._nodes = this._nodes;
        other._position = this._position;
        return other;
    }
    
    public void setDom(DOM dom, int node) {
        this._dom = dom;
        if ((dom instanceof MultiDOM)) {
            dom = ((MultiDOM)dom).getDTM(node);
        }
        if ((dom instanceof DOMEnhancedForDTM)) {
            this._enhancedDOM = (DOMEnhancedForDTM)dom;
        }
        else if ((dom instanceof DOMAdapter)) {
            DOM idom = ((DOMAdapter)dom).getDOMImpl();
            if ((idom instanceof DOMEnhancedForDTM)) {
                this._enhancedDOM = (DOMEnhancedForDTM)idom;
            }
        }
    }
    
    public KeyIndex.KeyIndexIterator getKeyIndexIterator(Object keyValue, boolean isKeyCall) {
        if ((keyValue instanceof DTMAxisIterator)) {
            return this.getKeyIndexIterator((DTMAxisIterator)keyValue, isKeyCall);
        }
        return this.getKeyIndexIterator(BasisLibrary.stringF(keyValue, this._dom), isKeyCall);
    }
    
    public KeyIndex.KeyIndexIterator getKeyIndexIterator(String keyValue, boolean isKeyCall) {
        return new KeyIndex.KeyIndexIterator(keyValue, isKeyCall);
    }
    
    public KeyIndex.KeyIndexIterator getKeyIndexIterator(DTMAxisIterator keyValue, boolean isKeyCall) {
        return new KeyIndex.KeyIndexIterator(keyValue, isKeyCall);
    }
    
    static {
        EMPTY_NODES = new IntegerArray(0);
    }
    
    public class KeyIndexIterator extends MultiValuedNodeHeapIterator
    {
        private IntegerArray _nodes;
        private DTMAxisIterator _keyValueIterator;
        private String _keyValue;
        private boolean _isKeyIterator;
        
        KeyIndexIterator(String keyValue, boolean isKeyIterator) {
            this._isKeyIterator = isKeyIterator;
            this._keyValue = keyValue;
        }
        
        KeyIndexIterator(DTMAxisIterator keyValues, boolean isKeyIterator) {
            this._keyValueIterator = keyValues;
            this._isKeyIterator = isKeyIterator;
        }
        
        protected IntegerArray lookupNodes(int root, String keyValue) {
            IntegerArray result = null;
            Map<String, IntegerArray> index = (Map)KeyIndex.this._rootToIndexMap.get(Integer.valueOf(root));
            if (!this._isKeyIterator) {
                StringTokenizer values = new StringTokenizer(keyValue, " \n\t");
                while (values.hasMoreElements()) {
                    String token = (String)values.nextElement();
                    IntegerArray nodes = null;
                    if (index != null) {
                        nodes = (IntegerArray)index.get(token);
                    }
                    if (nodes == null && KeyIndex.this._enhancedDOM != null && KeyIndex.this._enhancedDOM.hasDOMSource()) {
                        nodes = KeyIndex.this.getDOMNodeById(token);
                    }
                    if (nodes != null) {
                        if (result == null) {
                            result = (IntegerArray)nodes.clone();
                        }
                        else {
                            result.merge(nodes);
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
            else if (index != null) {
                result = (IntegerArray)index.get(keyValue);
            }
            return result;
        }
        
        @Override
        public DTMAxisIterator setStartNode(int node) {
            this._startNode = node;
            if (this._keyValueIterator != null) {
                this._keyValueIterator = this._keyValueIterator.setStartNode(node);
            }
            this.init();
            return super.setStartNode(node);
        }
        
        @Override
        public int next() {
            int nodeHandle;
            if (this._nodes != null) {
                if (this._position < this._nodes.cardinality()) {
                    nodeHandle = this.returnNode(this._nodes.at(this._position));
                }
                else {
                    nodeHandle = -1;
                }
            }
            else {
                nodeHandle = super.next();
            }
            return nodeHandle;
        }
        
        @Override
        public DTMAxisIterator reset() {
            if (this._nodes == null) {
                this.init();
            }
            else {
                super.reset();
            }
            return this.resetPosition();
        }
        
        @Override
        protected void init() {
            super.init();
            this._position = 0;
            int rootHandle = KeyIndex.this._dom.getAxisIterator(19).setStartNode(this._startNode).next();
            if (this._keyValueIterator == null) {
                this._nodes = this.lookupNodes(rootHandle, this._keyValue);
                if (this._nodes == null) {
                    this._nodes = KeyIndex.EMPTY_NODES;
                }
            }
            else {
                DTMAxisIterator keyValues = this._keyValueIterator.reset();
                int retrievedKeyValueIdx = 0;
                boolean foundNodes = false;
                this._nodes = null;
                for (int keyValueNode = keyValues.next(); keyValueNode != -1; keyValueNode = keyValues.next()) {
                    String keyValue = BasisLibrary.stringF(keyValueNode, KeyIndex.this._dom);
                    IntegerArray nodes = this.lookupNodes(rootHandle, keyValue);
                    if (nodes != null) {
                        if (!foundNodes) {
                            this._nodes = nodes;
                            foundNodes = true;
                        }
                        else {
                            if (this._nodes != null) {
                                this.addHeapNode(new KeyIndex.KeyIndexIterator.KeyIndexHeapNode(this._nodes));
                                this._nodes = null;
                            }
                            this.addHeapNode(new KeyIndex.KeyIndexIterator.KeyIndexHeapNode(nodes));
                        }
                    }
                }
                if (!foundNodes) {
                    this._nodes = KeyIndex.EMPTY_NODES;
                }
            }
        }
        
        @Override
        public int getLast() {
            return (this._nodes != null) ? this._nodes.cardinality() : super.getLast();
        }
        
        @Override
        public int getNodeByPosition(int position) {
            int node = -1;
            if (this._nodes != null) {
                if (position > 0) {
                    if (position <= this._nodes.cardinality()) {
                        this._position = position;
                        node = this._nodes.at(position - 1);
                    }
                    else {
                        this._position = this._nodes.cardinality();
                    }
                }
            }
            else {
                node = super.getNodeByPosition(position);
            }
            return node;
        }
        
        protected class KeyIndexHeapNode extends MultiValuedNodeHeapIterator.HeapNode
        {
            private IntegerArray _nodes;
            private int _position;
            private int _markPosition;
            
            KeyIndexHeapNode(IntegerArray nodes) {
                this._position = 0;
                this._markPosition = -1;
                this._nodes = nodes;
            }
            
            @Override
            public int step() {
                if (this._position < this._nodes.cardinality()) {
                    this._node = this._nodes.at(this._position);
                    ++this._position;
                }
                else {
                    this._node = -1;
                }
                return this._node;
            }
            
            @Override
            public MultiValuedNodeHeapIterator.HeapNode cloneHeapNode() {
                KeyIndex.KeyIndexIterator.KeyIndexHeapNode clone = (KeyIndex.KeyIndexIterator.KeyIndexHeapNode)super.cloneHeapNode();
                clone._nodes = this._nodes;
                clone._position = this._position;
                clone._markPosition = this._markPosition;
                return clone;
            }
            
            @Override
            public void setMark() {
                this._markPosition = this._position;
            }
            
            @Override
            public void gotoMark() {
                this._position = this._markPosition;
            }
            
            @Override
            public boolean isLessThan(MultiValuedNodeHeapIterator.HeapNode heapNode) {
                return this._node < heapNode._node;
            }
            
            @Override
            public MultiValuedNodeHeapIterator.HeapNode setStartNode(int node) {
                return this;
            }
            
            @Override
            public MultiValuedNodeHeapIterator.HeapNode reset() {
                this._position = 0;
                return this;
            }
        }
    }
}
